@@ -10,6 +10,12 @@ struct name_list
   struct name_list *next;
 };
 
+struct regex_list
+{
+  regex_t regex;
+  struct regex_list *next;
+};
+
 struct parameter_list
 {
   char *name;
@@ -24,6 +30,7 @@ struct parameter_list
   char *group;
   struct mount_entry *best_match;
   struct parameter_list *name_next;
+  struct parameter_list *name_prev;
   uintmax_t total, available, available_to_root, used,
     inodes_free, inodes_free_to_root, inodes_used, inodes_total;
   double dfree_pct, dused_pct;
@@ -32,12 +39,14 @@ struct parameter_list
 };
 
 void np_add_name (struct name_list **list, const char *name);
-int np_find_name (struct name_list *list, const char *name);
-int np_seen_name (struct name_list *list, const char *name);
+bool np_find_name (struct name_list *list, const char *name);
+bool np_seen_name (struct name_list *list, const char *name);
+int np_add_regex (struct regex_list **list, const char *regex, int cflags);
+bool np_find_regmatch (struct regex_list *list, const char *name);
 struct parameter_list *np_add_parameter(struct parameter_list **list, const char *name);
 struct parameter_list *np_find_parameter(struct parameter_list *list, const char *name);
 struct parameter_list *np_del_parameter(struct parameter_list *item, struct parameter_list *prev);
-  
+
 int search_parameter_list (struct parameter_list *list, const char *name);
-void np_set_best_match(struct parameter_list *desired, struct mount_entry *mount_list, int exact);
-int np_regex_match_mount_entry (struct mount_entry* me, regex_t* re);
+void np_set_best_match(struct parameter_list *desired, struct mount_entry *mount_list, bool exact);
+bool np_regex_match_mount_entry (struct mount_entry* me, regex_t* re);
